-----------------------------------------------------------------------------------------------
-- Client Lua Script for RosterThrottle
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Apollo"
require "Window"

local RosterThrottle = {}
local CarbineAddons = {
	["GuildContentRoster"] = "FullRedrawOfRoster",
	["Circles"] = "FullRedrawOfRoster",
	["NeighborList"] = "RefreshList",
	["FriendsList"] = "DrawList",
}
local sSocialContainer, uSocialContainerRef = "SocialPanelForm"
local AddonHooked = {}
local tOrigFunc = {}
local nRetries = 0

function RosterThrottle:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function RosterThrottle:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
	Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function RosterThrottle:OnLoad()
	-- never seen this retry more than once but a catchall mechanism for peace of mind.
 	if nRetries > 40 then Print('RosterThrottle: exceeded retry limit') return end
 	nRetries = nRetries + 1
	for addonName,funcName in pairs(CarbineAddons) do
		if not AddonHooked[addonName] then
			local carbineAddon = Apollo.GetAddon(addonName)
			if (carbineAddon) then
				tOrigFunc[addonName] = tOrigFunc[addonName] or {}
				tOrigFunc[addonName][funcName] = carbineAddon[funcName]
				carbineAddon[funcName] = function(...) 
					self.socialWindowCheck(addonName,...)
				end
				AddonHooked[addonName] = true
			else
				if not self.timer then
					self.timer = ApolloTimer.Create(5.0, false, "OnLoad", self)
				end
			end
		end
	end
end

function RosterThrottle.socialWindowCheck(addonName,...)
	local bSocialVisible = false
	if not uSocialContainerRef then
--		Print('getting window ref')
		local module = ...
		local wndMain = module.wndMain or module.tWndRefs.wndMain
		local wndTop = wndMain
		while (wndTop) do
			wndMain = wndTop
			wndTop = wndMain:GetParent()
		end
		if wndMain and wndMain:GetName() == sSocialContainer then
			uSocialContainerRef = wndMain
		end
	end
	bSocialVisible = uSocialContainerRef and uSocialContainerRef:IsShown()
	if bSocialVisible then
--		Print('updates passing through')
		local funcName = CarbineAddons[addonName]
		tOrigFunc[addonName][funcName](...)
	else
--		Print('updates blocked')
	end
end

local RosterThrottleInst = RosterThrottle:new()
RosterThrottleInst:Init()
